
module Multimedia.GStreamer where

import Foreign.C.String.Generalised
import Control.Monad.Trans (MonadIO, liftIO)
import Data.List (genericLength)
import Foreign.C.String (CString, newCString, peekCString)
import Foreign.C.Types (CInt)
import Data.Int
import Data.Word
import Foreign.Marshal.Alloc (malloc, free)
import Foreign.Marshal.Array (newArray)
import Foreign.Marshal.Utils (new)
import Foreign.Ptr (Ptr, nullPtr)
import Foreign.Storable (peek, poke)

#include <gst/gst.h>

-- Given up on types in here for now

data GstElement = GstElement

type State = #type GstElementState
type StateReturn = #type GstElementStateReturn
type Query = #type GstQueryType
type Format = #type GstFormat

state_playing, state_null :: State
state_playing = #const GST_STATE_PLAYING
state_null = #const GST_STATE_NULL

query_none, query_total, query_position, query_latency, query_jitter, query_start, query_segment_end, query_rate :: Query
query_none = #const GST_QUERY_NONE
query_total = #const GST_QUERY_TOTAL
query_position = #const GST_QUERY_POSITION
query_latency = #const GST_QUERY_LATENCY
query_jitter = #const GST_QUERY_JITTER 
query_start = #const GST_QUERY_START
query_segment_end = #const GST_QUERY_SEGMENT_END
query_rate = #const GST_QUERY_RATE

format_undefined, format_default, format_bytes, format_time, format_buffers, format_percent :: Format
format_undefined = #const GST_FORMAT_UNDEFINED
format_default = #const GST_FORMAT_DEFAULT
format_bytes = #const GST_FORMAT_BYTES
format_time = #const GST_FORMAT_TIME
format_buffers = #const GST_FORMAT_BUFFERS
format_percent = #const GST_FORMAT_PERCENT

foreign import ccall "static gst/gst.h gst_init"
    gst_init
    :: Ptr CInt -> Ptr CString -> IO ()
init :: MonadIO io => [String] -> io ()
init xs
 = liftIO $ do xs' <- mapM newCString xs
               xs'' <- newArray xs'
               l <- new (genericLength xs)
               gst_init l xs''
               free l
               free xs''
               mapM_ free xs'

foreign import ccall "static gst/gst.h gst_pipeline_new"
    gst_pipeline_new
    :: CString -> IO (Ptr a)
pipeline_new :: (IsCString str, MonadIO io) => str -> io (Ptr a)
pipeline_new s = liftIO $ withCString s gst_pipeline_new

foreign import ccall "static gst/gst.h gst_element_factory_make"
    gst_element_factory_make
    :: CString -> CString -> IO (Ptr a)
element_factory_make :: (IsCString str1, IsCString str2, MonadIO io)
                     => str1 -> str2 -> io (Ptr a)
element_factory_make fac_name name
    = liftIO $ do fac_name' <- toCString fac_name
                  name' <- toCString name
                  r <- gst_element_factory_make fac_name' name'
                  finishedWithCString name name'
                  finishedWithCString fac_name fac_name'
                  return r

foreign import ccall "static gst/gst.h gst_bin_add"
    gst_bin_add
    :: Ptr a -> Ptr b -> IO ()
bin_add :: MonadIO io => Ptr a -> Ptr b -> io ()
bin_add bin thing = liftIO $ gst_bin_add bin thing

foreign import ccall "static gst/gst.h gst_element_link"
    gst_element_link
    :: Ptr a -> Ptr b -> IO ()
element_link :: MonadIO io => Ptr a -> Ptr b -> io ()
element_link src dest = liftIO $ gst_element_link src dest

foreign import ccall "static gst/gst.h gst_element_get_state"
    gst_element_get_state
    :: Ptr a -> IO State
element_get_state :: MonadIO io => Ptr a -> io State
element_get_state el = liftIO $ gst_element_get_state el

foreign import ccall "static gst/gst.h gst_element_set_state"
    gst_element_set_state
    :: Ptr a -> State -> IO StateReturn
element_set_state :: MonadIO io => Ptr a -> State -> io Bool
element_set_state el state = liftIO $ do r <- gst_element_set_state el state
                                         return (r /= 0)

foreign import ccall "static gst/gst.h gst_bin_iterate"
    gst_bin_iterate
    :: Ptr a -> IO CInt
bin_iterate :: MonadIO io => Ptr a -> io Bool
bin_iterate bin = liftIO $ do r <- gst_bin_iterate bin
                              return (r /= 0)

foreign import ccall "static gst/gst.h gst_object_unref"
    gst_object_unref
    :: Ptr a -> IO ()
object_unref :: MonadIO io => Ptr a -> io ()
object_unref obj = liftIO $ gst_object_unref obj

foreign import ccall "static gst/gst.h gst_element_query"
    gst_element_query
    :: Ptr a -> Query -> Ptr Format -> Ptr Int64 -> IO Int
element_query :: MonadIO io
              => Ptr a -> Query -> Format -> io (Maybe (Format, Int64))
element_query el query_type format 
 = liftIO $ do i <- malloc
               format' <- new format
               r <- gst_element_query el query_type format' i
               res <- if r == 0
                      then return Nothing
                      else do format'' <- peek format'
                              i' <- peek i
                              return (Just (format'', i'))
               free format'
               free i
               return res

foreign import ccall "static gst/gst.h gst_tag_list_free"
    gst_tag_list_free
    :: Ptr a -> IO ()
tag_list_free :: MonadIO io => Ptr a -> io ()
tag_list_free tl = liftIO $ gst_tag_list_free tl

foreign import ccall "static gst/gst.h gst_tag_list_get_string"
    gst_tag_list_get_string
    :: Ptr a -> CString -> Ptr CString -> IO CInt
tag_list_get_string :: (IsCString str, MonadIO io)
                    => Ptr a -> str -> io (Maybe String)
tag_list_get_string tl field
 = liftIO $ do field' <- toCString field
               p_cs <- malloc
               poke p_cs nullPtr
               gst_tag_list_get_string tl field' p_cs
               finishedWithCString field field'
               cs <- peek p_cs
               free p_cs
               if cs == nullPtr
                then return Nothing
                else do s <- peekCString cs
                        return (Just s)

